@@ -50,6 +50,7 @@
 #include "shrpx_connect_blocker.h"
 #include "shrpx_api_downstream_connection.h"
 #include "shrpx_health_monitor_downstream_connection.h"
+#include "shrpx_log.h"
 #ifdef HAVE_SPDYLAY
 #include "shrpx_spdy_upstream.h"
 #endif // HAVE_SPDYLAY
@@ -960,7 +961,7 @@ uint32_t next_cycle(const WeightedPri &pri) {
 } // namespace
 
 std::unique_ptr<DownstreamConnection>
-ClientHandler::get_downstream_connection(Downstream *downstream) {
+ClientHandler::get_downstream_connection(int &err, Downstream *downstream) {
   size_t group_idx;
   auto &downstreamconf = *worker_->get_downstream_config();
   auto &routerconf = downstreamconf.router;
@@ -970,6 +971,8 @@ ClientHandler::get_downstream_connection(Downstream *downstream) {
 
   const auto &req = downstream->request();
 
+  err = 0;
+
   switch (faddr_->alt_mode) {
   case ALTMODE_API:
     return make_unique<APIDownstreamConnection>(worker_);
@@ -1008,6 +1011,15 @@ ClientHandler::get_downstream_connection(Downstream *downstream) {
     CLOG(INFO, this) << "Downstream address group_idx: " << group_idx;
   }
 
+  if (groups[group_idx]->shared_addr->redirect_if_not_tls && !conn_.tls.ssl) {
+    if (LOG_ENABLED(INFO)) {
+      CLOG(INFO, this) << "Downstream address group " << group_idx
+                       << " requires frontend TLS connection.";
+    }
+    err = SHRPX_ERR_TLS_REQUIRED;
+    return nullptr;
+  }
+
   auto &group = groups[group_idx];
   auto &shared_addr = group->shared_addr;
 
@@ -1079,6 +1091,7 @@ ClientHandler::get_downstream_connection(Downstream *downstream) {
       CLOG(INFO, this) << "No working downstream address found";
     }
 
+    err = -1;
     return nullptr;
   }
 
@@ -1091,6 +1104,7 @@ ClientHandler::get_downstream_connection(Downstream *downstream) {
     auto http2session = select_http2_session(group);
 
     if (http2session == nullptr) {
+      err = -1;
       return nullptr;
     }
 
