@@ -37,6 +37,7 @@
 #include "shrpx_http.h"
 #include "shrpx_worker.h"
 #include "shrpx_http2_session.h"
+#include "shrpx_log.h"
 #ifdef HAVE_MRUBY
 #include "shrpx_mruby.h"
 #endif // HAVE_MRUBY
@@ -270,16 +271,21 @@ int on_begin_headers_callback(nghttp2_session *session,
                          << frame->hd.stream_id;
   }
 
-  auto handler = upstream->get_client_handler();
+  upstream->on_start_request(frame);
 
-  auto downstream = make_unique<Downstream>(upstream, handler->get_mcpool(),
+  return 0;
+}
+} // namespace
+
+void Http2Upstream::on_start_request(const nghttp2_frame *frame) {
+  auto downstream = make_unique<Downstream>(this, handler_->get_mcpool(),
                                             frame->hd.stream_id);
-  nghttp2_session_set_stream_user_data(session, frame->hd.stream_id,
+  nghttp2_session_set_stream_user_data(session_, frame->hd.stream_id,
                                        downstream.get());
 
   downstream->reset_upstream_rtimer();
 
-  handler->repeat_read_timer();
+  handler_->repeat_read_timer();
 
   auto &req = downstream->request();
 
@@ -288,11 +294,16 @@ int on_begin_headers_callback(nghttp2_session *session,
   req.http_major = 2;
   req.http_minor = 0;
 
-  upstream->add_pending_downstream(std::move(downstream));
+  add_pending_downstream(std::move(downstream));
 
-  return 0;
+  ++num_requests_;
+
+  auto config = get_config();
+  auto &httpconf = config->http;
+  if (httpconf.max_requests <= num_requests_) {
+    start_graceful_shutdown();
+  }
 }
-} // namespace
 
 int Http2Upstream::on_request_headers(Downstream *downstream,
                                       const nghttp2_frame *frame) {
@@ -433,9 +444,25 @@ void Http2Upstream::start_downstream(Downstream *downstream) {
 void Http2Upstream::initiate_downstream(Downstream *downstream) {
   int rv;
 
-  auto dconn = handler_->get_downstream_connection(downstream);
-  if (!dconn ||
-      (rv = downstream->attach_downstream_connection(std::move(dconn))) != 0) {
+  auto dconn = handler_->get_downstream_connection(rv, downstream);
+  if (!dconn) {
+    if (rv == SHRPX_ERR_TLS_REQUIRED) {
+      rv = redirect_to_https(downstream);
+    } else {
+      rv = error_reply(downstream, 503);
+    }
+    if (rv != 0) {
+      rst_stream(downstream, NGHTTP2_INTERNAL_ERROR);
+    }
+
+    downstream->set_request_state(Downstream::CONNECT_FAIL);
+    downstream_queue_.mark_failure(downstream);
+
+    return;
+  }
+
+  rv = downstream->attach_downstream_connection(std::move(dconn));
+  if (rv != 0) {
     // downstream connection fails, send error page
     if (error_reply(downstream, 503) != 0) {
       rst_stream(downstream, NGHTTP2_INTERNAL_ERROR);
@@ -786,7 +813,11 @@ int send_data_callback(nghttp2_session *session, nghttp2_frame *frame,
 
   wb->append(PADDING.data(), padlen);
 
-  downstream->reset_upstream_wtimer();
+  if (body->rleft() == 0) {
+    downstream->disable_upstream_wtimer();
+  } else {
+    downstream->reset_upstream_wtimer();
+  }
 
   if (length > 0 && downstream->resume_read(SHRPX_NO_BUFFER, length) != 0) {
     return NGHTTP2_ERR_CALLBACK_FAILURE;
@@ -852,8 +883,6 @@ void Http2Upstream::submit_goaway() {
 }
 
 void Http2Upstream::check_shutdown() {
-  int rv;
-
   auto worker = handler_->get_worker();
 
   if (!worker->get_graceful_shutdown()) {
@@ -862,6 +891,15 @@ void Http2Upstream::check_shutdown() {
 
   ev_prepare_stop(handler_->get_loop(), &prep_);
 
+  start_graceful_shutdown();
+}
+
+void Http2Upstream::start_graceful_shutdown() {
+  int rv;
+  if (ev_is_active(&shutdown_timer_)) {
+    return;
+  }
+
   rv = nghttp2_submit_shutdown_notice(session_);
   if (rv != 0) {
     ULOG(FATAL, this) << "nghttp2_submit_shutdown_notice() failed: "
@@ -944,7 +982,8 @@ Http2Upstream::Http2Upstream(ClientHandler *handler)
                         !get_config()->http2_proxy),
       handler_(handler),
       session_(nullptr),
-      max_buffer_size_(MAX_BUFFER_SIZE) {
+      max_buffer_size_(MAX_BUFFER_SIZE),
+      num_requests_(0) {
   int rv;
 
   auto config = get_config();
@@ -1396,6 +1435,7 @@ ssize_t downstream_data_read_callback(nghttp2_session *session,
   }
 
   if (nread == 0 && ((*data_flags) & NGHTTP2_DATA_FLAG_EOF) == 0) {
+    downstream->disable_upstream_wtimer();
     return NGHTTP2_ERR_DEFERRED;
   }
 
@@ -1468,6 +1508,10 @@ int Http2Upstream::send_reply(Downstream *downstream, const uint8_t *body,
 
   downstream->set_response_state(Downstream::MSG_COMPLETE);
 
+  if (data_prd_ptr) {
+    downstream->reset_upstream_wtimer();
+  }
+
   return 0;
 }
 
@@ -1510,6 +1554,8 @@ int Http2Upstream::error_reply(Downstream *downstream,
     return -1;
   }
 
+  downstream->reset_upstream_wtimer();
+
   return 0;
 }
 
@@ -1716,6 +1762,10 @@ int Http2Upstream::on_downstream_header_complete(Downstream *downstream) {
     return -1;
   }
 
+  if (data_prdptr) {
+    downstream->reset_upstream_wtimer();
+  }
+
   return 0;
 }
 
@@ -1791,6 +1841,52 @@ int Http2Upstream::on_downstream_abort_request(Downstream *downstream,
   return 0;
 }
 
+int Http2Upstream::on_downstream_abort_request_with_https_redirect(
+    Downstream *downstream) {
+  int rv;
+
+  rv = redirect_to_https(downstream);
+  if (rv != 0) {
+    return -1;
+  }
+
+  handler_->signal_write();
+  return 0;
+}
+
+int Http2Upstream::redirect_to_https(Downstream *downstream) {
+  auto &req = downstream->request();
+  if (req.method == HTTP_CONNECT || req.scheme != "http") {
+    return error_reply(downstream, 400);
+  }
+
+  auto authority = util::extract_host(req.authority);
+  if (authority.empty()) {
+    return error_reply(downstream, 400);
+  }
+
+  auto &balloc = downstream->get_block_allocator();
+  auto config = get_config();
+  auto &httpconf = config->http;
+
+  StringRef loc;
+  if (httpconf.redirect_https_port == StringRef::from_lit("443")) {
+    loc = concat_string_ref(balloc, StringRef::from_lit("https://"), authority,
+                            req.path);
+  } else {
+    loc = concat_string_ref(balloc, StringRef::from_lit("https://"), authority,
+                            StringRef::from_lit(":"),
+                            httpconf.redirect_https_port, req.path);
+  }
+
+  auto &resp = downstream->response();
+  resp.http_status = 308;
+  resp.fs.add_header_token(StringRef::from_lit("location"), loc, false,
+                           http2::HD_LOCATION);
+
+  return send_reply(downstream, nullptr, 0);
+}
+
 int Http2Upstream::consume(int32_t stream_id, size_t len) {
   int rv;
 
@@ -1829,7 +1925,8 @@ int Http2Upstream::on_timeout(Downstream *downstream) {
                      << downstream->get_stream_id();
   }
 
-  rst_stream(downstream, NGHTTP2_NO_ERROR);
+  rst_stream(downstream, NGHTTP2_INTERNAL_ERROR);
+  handler_->signal_write();
 
   return 0;
 }
@@ -1878,6 +1975,8 @@ int Http2Upstream::on_downstream_reset(Downstream *downstream, bool no_retry) {
 
   std::unique_ptr<DownstreamConnection> dconn;
 
+  rv = 0;
+
   if (no_retry || downstream->no_more_retry()) {
     goto fail;
   }
@@ -1885,7 +1984,7 @@ int Http2Upstream::on_downstream_reset(Downstream *downstream, bool no_retry) {
   // downstream connection is clean; we can retry with new
   // downstream connection.
 
-  dconn = handler_->get_downstream_connection(downstream);
+  dconn = handler_->get_downstream_connection(rv, downstream);
   if (!dconn) {
     goto fail;
   }
@@ -1903,7 +2002,12 @@ int Http2Upstream::on_downstream_reset(Downstream *downstream, bool no_retry) {
   return 0;
 
 fail:
-  if (on_downstream_abort_request(downstream, 503) != 0) {
+  if (rv == SHRPX_ERR_TLS_REQUIRED) {
+    rv = on_downstream_abort_request_with_https_redirect(downstream);
+  } else {
+    rv = on_downstream_abort_request(downstream, 503);
+  }
+  if (rv != 0) {
     rst_stream(downstream, NGHTTP2_INTERNAL_ERROR);
   }
   downstream->pop_downstream_connection();
