@@ -27,6 +27,7 @@
 #include <algorithm>
 
 #include "shrpx_config.h"
+#include "shrpx_log.h"
 
 namespace shrpx {
 
@@ -35,7 +36,9 @@ RNode::RNode() : s(nullptr), len(0), index(-1) {}
 RNode::RNode(const char *s, size_t len, size_t index)
     : s(s), len(len), index(index) {}
 
-Router::Router() : root_{} {}
+Router::Router() : balloc_(1024, 1024), root_{} {}
+
+Router::~Router() {}
 
 namespace {
 RNode *find_next_node(const RNode *node, char c) {
@@ -62,11 +65,12 @@ void add_next_node(RNode *node, std::unique_ptr<RNode> new_node) {
 
 void Router::add_node(RNode *node, const char *pattern, size_t patlen,
                       size_t index) {
-  auto new_node = make_unique<RNode>(pattern, patlen, index);
+  auto pat = make_string_ref(balloc_, StringRef{pattern, patlen});
+  auto new_node = make_unique<RNode>(pat.c_str(), pat.size(), index);
   add_next_node(node, std::move(new_node));
 }
 
-bool Router::add_route(const StringRef &pattern, size_t index) {
+size_t Router::add_route(const StringRef &pattern, size_t index) {
   auto node = &root_;
   size_t i = 0;
 
@@ -74,7 +78,7 @@ bool Router::add_route(const StringRef &pattern, size_t index) {
     auto next_node = find_next_node(node, pattern[i]);
     if (next_node == nullptr) {
       add_node(node, pattern.c_str() + i, pattern.size() - i, index);
-      return true;
+      return index;
     }
 
     node = next_node;
@@ -90,11 +94,11 @@ bool Router::add_route(const StringRef &pattern, size_t index) {
       if (slen == node->len) {
         // Complete match
         if (node->index != -1) {
-          // Don't allow duplicate
-          return false;
+          // Return the existing index for duplicates.
+          return node->index;
         }
         node->index = index;
-        return true;
+        return index;
       }
 
       if (slen > node->len) {
@@ -119,7 +123,7 @@ bool Router::add_route(const StringRef &pattern, size_t index) {
 
       if (slen == j) {
         node->index = index;
-        return true;
+        return index;
       }
     }
 
@@ -128,7 +132,7 @@ bool Router::add_route(const StringRef &pattern, size_t index) {
     assert(pattern.size() > i);
     add_node(node, pattern.c_str() + i, pattern.size() - i, index);
 
-    return true;
+    return index;
   }
 }
 
@@ -276,6 +280,81 @@ ssize_t Router::match(const StringRef &host, const StringRef &path) const {
   return node->index;
 }
 
+ssize_t Router::match(const StringRef &s) const {
+  const RNode *node;
+  size_t offset;
+
+  node = match_complete(&offset, &root_, std::begin(s), std::end(s));
+  if (node == nullptr) {
+    return -1;
+  }
+
+  if (node->len != offset) {
+    return -1;
+  }
+
+  return node->index;
+}
+
+namespace {
+const RNode *match_prefix(size_t *nread, const RNode *node, const char *first,
+                          const char *last) {
+  if (first == last) {
+    return nullptr;
+  }
+
+  auto p = first;
+
+  for (;;) {
+    auto next_node = find_next_node(node, *p);
+    if (next_node == nullptr) {
+      return nullptr;
+    }
+
+    node = next_node;
+
+    auto n = std::min(node->len, static_cast<size_t>(last - p));
+    if (memcmp(node->s, p, n) != 0) {
+      return nullptr;
+    }
+
+    p += n;
+
+    if (p != last) {
+      if (node->index != -1) {
+        *nread = p - first;
+        return node;
+      }
+      continue;
+    }
+
+    if (node->len == n) {
+      *nread = p - first;
+      return node;
+    }
+
+    return nullptr;
+  }
+}
+} // namespace
+
+ssize_t Router::match_prefix(size_t *nread, const RNode **last_node,
+                             const StringRef &s) const {
+  if (*last_node == nullptr) {
+    *last_node = &root_;
+  }
+
+  auto node =
+      ::shrpx::match_prefix(nread, *last_node, std::begin(s), std::end(s));
+  if (node == nullptr) {
+    return -1;
+  }
+
+  *last_node = node;
+
+  return node->index;
+}
+
 namespace {
 void dump_node(const RNode *node, int depth) {
   fprintf(stderr, "%*ss='%.*s', len=%zu, index=%zd\n", depth, "",
