@@ -52,6 +52,7 @@
 #include "shrpx_memcached_request.h"
 #include "shrpx_process.h"
 #include "shrpx_ssl.h"
+#include "shrpx_log.h"
 #include "util.h"
 #include "app_helper.h"
 #include "template.h"
@@ -71,6 +72,12 @@ void drop_privileges(
   auto config = get_config();
 
   if (getuid() == 0 && config->uid != 0) {
+#ifdef HAVE_NEVERBLEED
+    if (nb) {
+      neverbleed_setuidgid(nb, config->user.c_str(), 1);
+    }
+#endif // HAVE_NEVERBLEED
+
     if (initgroups(config->user.c_str(), config->gid) != 0) {
       auto error = errno;
       LOG(FATAL) << "Could not change supplementary groups: "
@@ -93,11 +100,6 @@ void drop_privileges(
       LOG(FATAL) << "Still have root privileges?";
       exit(EXIT_FAILURE);
     }
-#ifdef HAVE_NEVERBLEED
-    if (nb) {
-      neverbleed_setuidgid(nb, config->user.c_str(), 1);
-    }
-#endif // HAVE_NEVERBLEED
   }
 }
 } // namespace
@@ -121,9 +123,9 @@ void graceful_shutdown(ConnectionHandler *conn_handler) {
 
   conn_handler->graceful_shutdown_worker();
 
-  if (get_config()->num_worker == 1) {
-    if (conn_handler->get_single_worker()->get_worker_stat()->num_connections ==
-        0) {
+  auto single_worker = conn_handler->get_single_worker();
+  if (single_worker) {
+    if (single_worker->get_worker_stat()->num_connections == 0) {
       ev_break(conn_handler->get_loop());
     }
 
@@ -136,12 +138,13 @@ namespace {
 void reopen_log(ConnectionHandler *conn_handler) {
   LOG(NOTICE) << "Reopening log files: worker process (thread main)";
 
-  (void)reopen_log_files();
-  redirect_stderr_to_errorlog();
+  auto config = get_config();
+  auto &loggingconf = config->logging;
 
-  if (get_config()->num_worker > 1) {
-    conn_handler->worker_reopen_log_files();
-  }
+  (void)reopen_log_files(loggingconf);
+  redirect_stderr_to_errorlog(loggingconf);
+
+  conn_handler->worker_reopen_log_files();
 }
 } // namespace
 
@@ -398,7 +401,9 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
   std::array<char, STRERROR_BUFSIZE> errbuf;
   (void)errbuf;
 
-  if (reopen_log_files() != 0) {
+  auto config = get_config();
+
+  if (reopen_log_files(config->logging) != 0) {
     LOG(FATAL) << "Failed to open log file";
     return -1;
   }
@@ -415,8 +420,6 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
 
   ConnectionHandler conn_handler(loop, gen);
 
-  auto config = get_config();
-
   for (auto &addr : config->conn.listener.addrs) {
     conn_handler.add_acceptor(make_unique<AcceptHandler>(&addr, &conn_handler));
   }
@@ -449,7 +452,7 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
   MemchunkPool mcpool;
 
   ev_timer renew_ticket_key_timer;
-  if (ssl::upstream_tls_enabled()) {
+  if (ssl::upstream_tls_enabled(config->conn)) {
     auto &ticketconf = config->tls.ticket;
     auto &memcachedconf = ticketconf.memcached;
 
@@ -503,7 +506,7 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
     }
   }
 
-  if (config->num_worker == 1) {
+  if (config->single_thread) {
     rv = conn_handler.create_single_worker();
     if (rv != 0) {
       return -1;
@@ -548,7 +551,7 @@ int worker_process_event_loop(WorkerProcessConfig *wpconf) {
   ipcev.data = &conn_handler;
   ev_io_start(loop, &ipcev);
 
-  if (ssl::upstream_tls_enabled() && !config->tls.ocsp.disabled) {
+  if (ssl::upstream_tls_enabled(config->conn) && !config->tls.ocsp.disabled) {
     conn_handler.proceed_next_cert_ocsp();
   }
 
