@@ -312,6 +312,8 @@ int main(int argc _U_, char *argv[] _U_) {
                    test_nghttp2_session_removed_closed_stream) ||
       !CU_add_test(pSuite, "session_pause_data",
                    test_nghttp2_session_pause_data) ||
+      !CU_add_test(pSuite, "session_no_closed_streams",
+                   test_nghttp2_session_no_closed_streams) ||
       !CU_add_test(pSuite, "http_mandatory_headers",
                    test_nghttp2_http_mandatory_headers) ||
       !CU_add_test(pSuite, "http_content_length",
